import * as knex from 'knex';
import { ModelDefinition, SerializationOptions } from './definition';
export declare class BaseMapper {
    private knexBuilder;
    protected options: SerializationOptions;
    constructor(knexBuilder: knex.QueryInterface, options: SerializationOptions);
    selectAllFrom<T>(model: ModelDefinition<T>): SelectQuery<T>;
    updateWith<T extends U, U>(model: ModelDefinition<T>, data: T): UpdateQuery<T>;
    insertInto<T extends U, U>(model: ModelDefinition<T>, data: T): InsertQuery<T>;
    truncate<T>(model: ModelDefinition<T>): Promise<void>;
    tryFindOneByKey<T extends U, U>(model: ModelDefinition<T>, key: U): Promise<T>;
    findOneByKey<T extends U, U>(model: ModelDefinition<T>, key: U): Promise<T>;
}
export declare class Mapper extends BaseMapper {
    private knexClient;
    constructor(knexClient: knex, options: SerializationOptions);
    transaction<T>(callback: (transactionMapper: BaseMapper) => Promise<T>): Promise<T>;
}
export interface WhereClause {
    operator: string;
    operands: WhereClause[];
}
export declare class BaseQuery {
    protected knexQuery: knex.QueryBuilder;
    constructor(knexQuery: knex.QueryBuilder);
    getKnexQuery(): knex.QueryBuilder;
}
export declare class WhereQuery extends BaseQuery {
    constructor(knexQuery: knex.QueryBuilder);
    where(clause: WhereClause): void;
    whereEqual<T>(attribute: T, value: T): this;
    whereLess<T>(attribute: T, value: T): this;
    whereLessOrEqual<T>(attribute: T, value: T): this;
    whereGreater<T>(attribute: T, value: T): this;
    whereGreaterOrEqual<T>(attribute: T, value: T): this;
    private whereOperator<T>(attribute, operator, value);
}
export declare class SelectQuery<ResultType> extends WhereQuery {
    private resultModel;
    private serializationOptions;
    constructor(resultModel: ModelDefinition<ResultType>, knexQuery: knex.QueryBuilder, serializationOptions: SerializationOptions);
    forUpdate(): this;
    limit(count: number): this;
    orderBy(attribute: any, direction: 'asc' | 'desc'): this;
    getOne(): Promise<ResultType>;
    execute(): Promise<ResultType[]>;
    then<TResult>(onfulfilled?: (value: ResultType[]) => TResult | PromiseLike<TResult>, onrejected?: (reason: any) => TResult | PromiseLike<TResult>): PromiseLike<TResult>;
}
export declare class UpdateQuery<UpdateDataType> extends WhereQuery implements PromiseLike<void> {
    private model;
    private serializationOptions;
    constructor(model: ModelDefinition<UpdateDataType>, knexQuery: knex.QueryBuilder, serializationOptions: SerializationOptions);
    execute(): Promise<void>;
    then<TResult>(onfulfilled?: () => TResult | PromiseLike<TResult>, onrejected?: (reason: any) => TResult | PromiseLike<TResult>): PromiseLike<TResult>;
}
export declare class InsertQuery<InsertDataType> extends BaseQuery implements PromiseLike<void> {
    private model;
    private serializationOptions;
    constructor(model: ModelDefinition<InsertDataType>, knexQuery: knex.QueryBuilder, serializationOptions: SerializationOptions);
    execute(): Promise<void>;
    then<TResult>(onfulfilled?: () => TResult | PromiseLike<TResult>, onrejected?: (reason: any) => TResult | PromiseLike<TResult>): PromiseLike<TResult>;
}
