export interface Metadata {
    tableName: string;
    attributes: {
        [attributeName: string]: AttributeDefinition;
    };
    fields: {
        [fieldName: string]: AttributeDefinition;
    };
}
export interface MetadataDefinition {
    __metadata: Metadata;
}
export declare type ModelDefinition<InstanceType> = MetadataDefinition & InstanceType;
export declare function defineModel<InstanceType>(tableName: string, prototypeDefinition: InstanceType): ModelDefinition<InstanceType>;
export declare function defineString(options?: AttributeOptions): string;
export declare function defineNumber(options?: AttributeOptions): number;
export declare function defineJson<Subtype>(options?: AttributeOptions): Subtype;
export declare function defineDatetime(options?: AttributeOptions): Date;
export declare type DataMapperOptions = SerializationOptions;
export interface AttributeOptions {
    fieldName?: string;
}
export declare type DataType = 'number' | 'string' | 'json' | 'datetime';
export interface AttributeDefinition {
    dataType: DataType;
    attributeName: string;
    fieldName?: string;
}
export interface SerializationOptions {
    stringifyJson?: boolean;
}
export declare function getTableName<T>(definition: ModelDefinition<T>): string;
export declare function getAttributes<T>(definition: ModelDefinition<T>): string[];
export declare function getFieldNames<T>(definition: ModelDefinition<T>): string[];
export declare function getDataAttributes(data: any): string[];
export declare function serializeData<ModelDataType extends DataType, DataType>(model: ModelDefinition<ModelDataType>, data: DataType, serializationOptions: SerializationOptions): any;
export declare function deserializeData<ModelDataType>(model: ModelDefinition<ModelDataType>, fieldData: any, serializationOptions: SerializationOptions): ModelDataType;
