"use strict";
function defineModel(tableName, prototypeDefinition) {
    let definition = {
        __metadata: {
            tableName: tableName,
            attributes: {},
            fields: {}
        }
    };
    let metadata = definition.__metadata;
    let attributeNames = getDataAttributes(prototypeDefinition);
    for (let attributeName of attributeNames) {
        let attributeDefinition = {
            dataType: prototypeDefinition[attributeName]._type,
            attributeName: attributeName,
            fieldName: prototypeDefinition[attributeName].fieldName || attributeName
        };
        // TODO: Check for duplicate field names
        //definition.prototypeDefinition[attributeName] = attributeDefinition;
        metadata.attributes[attributeName] = attributeDefinition;
        metadata.fields[attributeDefinition.fieldName] = attributeDefinition;
    }
    return Object.assign(definition, metadata.attributes);
}
exports.defineModel = defineModel;
function defineString(options) {
    let definition = options || {};
    definition._type = 'string';
    return definition;
}
exports.defineString = defineString;
function defineNumber(options) {
    let definition = options || {};
    definition._type = 'number';
    return definition;
}
exports.defineNumber = defineNumber;
function defineJson(options) {
    let definition = options || {};
    definition._type = 'json';
    return definition;
}
exports.defineJson = defineJson;
function defineDatetime(options) {
    let definition = options || {};
    definition._type = 'datetime';
    return definition;
}
exports.defineDatetime = defineDatetime;
function getTableName(definition) {
    return definition.__metadata.tableName;
}
exports.getTableName = getTableName;
function getAttributes(definition) {
    return Object.keys(definition.__metadata.attributes);
}
exports.getAttributes = getAttributes;
function getFieldNames(definition) {
    return Object.keys(definition.__metadata.fields);
}
exports.getFieldNames = getFieldNames;
function getDataAttributes(data) {
    return Object.keys(data);
}
exports.getDataAttributes = getDataAttributes;
function serializeData(model, data, serializationOptions) {
    let attributeNames = getDataAttributes(data);
    let fieldData = {};
    for (let attributeName of attributeNames) {
        let attributeDefinition = model.__metadata.attributes[attributeName];
        if (serializationOptions.stringifyJson && attributeDefinition.dataType === 'json') {
            fieldData[attributeDefinition.fieldName] = JSON.stringify(data[attributeName]);
        }
        else {
            fieldData[attributeDefinition.fieldName] = data[attributeName];
        }
    }
    return fieldData;
}
exports.serializeData = serializeData;
function deserializeData(model, fieldData, serializationOptions) {
    let data = {};
    let fieldNames = getDataAttributes(fieldData);
    for (let fieldName of fieldNames) {
        let attributeDefinition = model.__metadata.fields[fieldName];
        let fieldValue = fieldData[attributeDefinition.fieldName];
        if (serializationOptions.stringifyJson && attributeDefinition.dataType === 'json') {
            data[attributeDefinition.attributeName] = JSON.parse(fieldValue);
        }
        else if (attributeDefinition.dataType === 'datetime' && typeof fieldValue === 'number') {
            data[attributeDefinition.attributeName] = new Date(fieldData[attributeDefinition.fieldName]);
        }
        else {
            data[attributeDefinition.attributeName] = fieldValue;
        }
    }
    return data;
}
exports.deserializeData = deserializeData;
//# sourceMappingURL=definition.js.map